<?php

namespace Drupal\feeds_filemaker\Feeds\Parser;

use Drupal\feeds\FeedInterface;
use Drupal\feeds\Plugin\Type\Parser\ParserInterface;
use Drupal\feeds\Result\FetcherResultInterface;
use Drupal\feeds\Result\ParserResult;
use Drupal\feeds\Plugin\Type\PluginBase;
use Drupal\feeds\StateInterface;
use Drupal\feeds_filemaker\Feeds\Item\FileMakerItem;

/**
 * Defines a base FileMaker JSON parser.
 */
abstract class FileMakerJsonParserBase extends PluginBase implements ParserInterface {

  /**
   * {@inheritdoc}
   */
  public function parse(FeedInterface $feed, FetcherResultInterface $fetcher_result, StateInterface $state): ParserResult {
    $raw = $fetcher_result->getRaw();
    $parsed_data = json_decode($raw, TRUE);
    $result = new ParserResult();

    if (isset($parsed_data['response']['data'])) {
      foreach ($parsed_data['response']['data'] as $item) {
        $result->addItem($this->parseItem($item));
      }
    }

    return $result;
  }

  abstract protected function parseItem(array $data): FileMakerItem;

  abstract public function getMappingSources(): array;

  abstract public function getSupportedCustomSourcePlugins(): array;

}
